@@ -11,8 +11,7 @@ from typing import TYPE_CHECKING, Any, Iterable, Type, cast
 from urllib.parse import urlparse
 
 import tomlkit
-from pythonfinder import Finder
-from pythonfinder.environment import PYENV_INSTALLED, PYENV_ROOT
+from findpython import Finder
 
 from pdm import termui
 from pdm._types import Source
@@ -47,13 +46,22 @@ if TYPE_CHECKING:
     from pdm.resolver.providers import BaseProvider
 
 
+PYENV_ROOT = os.path.expanduser(os.getenv("PYENV_ROOT", "~/.pyenv"))
+
+
 class Project:
-    """Core project class"""
+    """Core project class.
+
+    Args:
+        core: The core instance.
+        root_path: The root path of the project.
+        is_global: Whether the project is global.
+        global_config: The path to the global config file.
+    """
 
     PYPROJECT_FILENAME = "pyproject.toml"
     DEPENDENCIES_RE = re.compile(r"(?:(.+?)-)?dependencies")
     LOCKFILE_VERSION = "3.1"
-    GLOBAL_PROJECT = Path.home() / ".pdm" / "global-project"
 
     def __init__(
         self,
@@ -69,22 +77,27 @@ class Project:
         self.core = core
 
         if global_config is None:
-            global_config = self.GLOBAL_PROJECT.with_name("config.toml")
+            global_config = Path.home() / ".pdm/config.toml"
         self.global_config = Config(Path(global_config), is_global=True)
+        global_project = Path(self.global_config["global_project.path"])
 
         if root_path is None:
             root_path = (
                 find_project_root(max_depth=self.global_config["project_max_depth"])
                 if not is_global
-                else self.GLOBAL_PROJECT
+                else global_project
             )
-        if not is_global and root_path is None and self.global_config["auto_global"]:
+        if (
+            not is_global
+            and root_path is None
+            and self.global_config["global_project.fallback"]
+        ):
             self.core.ui.echo(
                 "Project is not found, fallback to the global project",
                 fg="yellow",
                 err=True,
             )
-            root_path = self.GLOBAL_PROJECT
+            root_path = global_project
             is_global = True
 
         self.root = Path(root_path or "").absolute()
@@ -120,6 +133,10 @@ class Project:
             return {}
         return data.setdefault("tool", {}).setdefault("pdm", {})
 
+    @property
+    def name(self) -> str | None:
+        return self.meta.get("name")
+
     @property
     def lockfile(self) -> dict:
         if not self._lockfile:
@@ -163,19 +180,19 @@ class Project:
     @property
     def python_executable(self) -> str:
         """For backward compatibility"""
-        return self.python.executable
+        return str(self.python.executable)
 
     def resolve_interpreter(self) -> PythonInfo:
         """Get the Python interpreter path."""
         config = self.config
         if config.get("python.path") and not os.getenv("PDM_IGNORE_SAVED_PYTHON"):
             saved_path = config["python.path"]
-            try:
-                python = PythonInfo.from_path(saved_path)
-                if self.python_requires.contains(str(python.version)):
-                    return python
-            except (ValueError, FileNotFoundError):
-                self.project_config.pop("python.path", None)
+            python = PythonInfo.from_path(saved_path)
+            if python.valid and self.python_requires.contains(
+                str(python.version), True
+            ):
+                return python
+            self.project_config.pop("python.path", None)
         if os.name == "nt":
             suffix = ".exe"
             scripts = "Scripts"
@@ -186,12 +203,16 @@ class Project:
         # Resolve virtual environments from env-vars
         virtual_env = os.getenv("VIRTUAL_ENV", os.getenv("CONDA_PREFIX"))
         if config["python.use_venv"] and virtual_env:
-            return PythonInfo.from_path(
+            python = PythonInfo.from_path(
                 os.path.join(virtual_env, scripts, f"python{suffix}")
             )
+            if python.valid:
+                return python
 
         for py_version in self.find_interpreters():
-            if self.python_requires.contains(str(py_version.version)):
+            if py_version.valid and self.python_requires.contains(
+                str(py_version.version), True
+            ):
                 self.python = py_version
                 return py_version
 
@@ -207,7 +228,7 @@ class Project:
             env = GlobalEnvironment(self)
             # Rewrite global project's python requires to be
             # compatible with the exact version
-            env.python_requires = PySpecSet(f"=={self.python.version}")
+            env.python_requires = PySpecSet.equal_to(self.python.version)
             return env
         if self.config["python.use_venv"] and get_venv_like_prefix(
             self.python.executable
@@ -576,7 +597,7 @@ dependencies = ["pip", "setuptools", "wheel"]
         python: str | Path | None = None
 
         if not python_spec:
-            if config.get("python.use_pyenv", True) and PYENV_INSTALLED:
+            if config.get("python.use_pyenv", True) and os.path.exists(PYENV_ROOT):
                 pyenv_shim = os.path.join(PYENV_ROOT, "shims", "python3")
                 if os.name == "nt":
                     pyenv_shim += ".bat"
@@ -604,9 +625,9 @@ dependencies = ["pip", "setuptools", "wheel"]
                         yield PythonInfo.from_path(python)
                 return
             args = [int(v) for v in python_spec.split(".") if v != ""]
-        finder = Finder()
-        for entry in finder.find_all_python_versions(*args):
-            yield PythonInfo.from_python_version(entry.py_version)
+        finder = Finder(resolve_symlinks=True)
+        for entry in finder.find_all(*args):
+            yield PythonInfo(entry)
         if not python_spec:
             this_python = getattr(sys, "_base_executable", sys.executable)
             yield PythonInfo.from_path(this_python)
